# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import GameState

BASE_DIMENSION = 1

class HexapawnGameState(GameState):
    def reset(self):
        GameState.reset(self)
        # fill starting "rows"
        nrows = self.game_params.get('fillRows', 1)
        for p in self.itercoords():
            val = 0
            if p[BASE_DIMENSION]<nrows: val = 1
            elif (self.grid.shape[BASE_DIMENSION]-p[BASE_DIMENSION])<=nrows: val = 2
            self.grid[p] = val
            
    def has_scores(self):
        return False

    def can_move_forward(self):
       return 'forward' in self.game_params.get('moves')

    def can_move_diagonally(self):
       return 'diagonal' in self.game_params.get('moves')

    def can_move_sideways(self):
       return 'side' in self.game_params.get('moves')
        
    def can_capture_forward(self):
       return 'forward' in self.game_params.get('captures')

    def can_capture_diagonally(self):
       return 'diagonal' in self.game_params.get('captures')

    def can_capture_sideways(self):
       return 'side' in self.game_params.get('captures')
        
    def moves_from_position(self, pos, pnum, delta):
        # get forward position
        fwdpos = tuple( (val if i!=BASE_DIMENSION else val+delta) for i,val in enumerate(pos) )
        # get all neighbors, check for move to empty space or capture enemy
        for np,nval in self.enumerate_neighbors(pos):
            if nval==pnum: continue
            good = False
            if np==fwdpos:
                good = self.can_move_forward() if nval==0 else self.can_capture_forward()
            elif pos[BASE_DIMENSION]+delta==np[BASE_DIMENSION]:
                good = self.can_move_diagonally() if nval==0 else self.can_capture_diagonally()
            elif pos[BASE_DIMENSION]==np[BASE_DIMENSION]:
                good = self.can_move_sideways() if nval==0 else self.can_capture_sideways()
            
            if good: yield np
        
    def successors(self):
        pnum = self.current_player
        delta = 1 if pnum==1 else -1
        for p in self.enumerate_coords_with_value(pnum):
            for np in self.moves_from_position(p, pnum, delta):
                nextgrid = self.grid.copy()
                nextgrid[p]  = 0
                nextgrid[np] = pnum
                yield ( (p,np), self.next_state_with_grid(nextgrid) )
                
    def is_game_over(self):
        #print 'is_game_over', self.previous_player()
        # only need to check previous player, wins if piece is at coord 0 (player 2) or max-1 (player 1)
        prevplayer = self.previous_player()
        wincoord = (0 if prevplayer==2 else self.grid.shape[BASE_DIMENSION]-1)
        for p in self.enumerate_coords_with_value(prevplayer):
            if p[BASE_DIMENSION]==wincoord: return True
        # game also over if no valid moves
        return not any(self.successors())
    
    def winning_player(self):
        return self.previous_player()

    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        return PGPieceAnimation.AnimationGroup(PGPieceAnimation.move_with_capture_animations(move[0], move[1]))
   
